-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                  "EskaQuestTracker.WorldQuests"                  "1.0.0"
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
IsWorldQuest       = QuestUtils_IsQuestWorldQuest
GetTaskInfo        = GetTaskInfo
IsInInstance       = IsInInstance
-- ======================[[ DATA ]]========================================== --
WORLD_QUESTS_CACHE = {}
-- @NOTE: The blacklist is used to hide the weekly world quest (e.g :
-- 2v2, 3v3 weekling quest)
WORLD_QUESTS_BLACKLIST = {
  [44891] = true, -- 2v2 Weekly quest
  [44908] = true, -- 3v3 Weekly quest
  [44909] = true, -- RBG Weekly quest
}
-- ========================================================================== --


function OnLoad(self)
  -- Check if the player is a world quest zone
  self._Enabled = self:HasWorldQuest()
end

function OnEnable(self)
  Debug("World quests module is enabled")
  if not _WorldQuestBlock then
    _WorldQuestBlock = WorldQuestBlock()
    _Addon:RegisterBlock(_WorldQuestBlock)
  end

  _WorldQuestBlock.isActive = true
  self:LoadWorldQuests()
end

function OnDisable(self)
  Debug("World quests module is disabled")
  if _WorldQuestBlock then
    _WorldQuestBlock.isActive = false
  end
end




--[[
__SystemEvent__()
function PLAYER_ENTERING_WORLD()
    local inInstance, type = IsInInstance()
    if inInstance and type == "party" then
      _M:LoadWorldQuests()
    else
      _M:ClearWorldQuests()
      _M:LoadWorldQuests()
    end
end--]]


__EnableOnCondition__ "QUEST_ACCEPTED" "PLAYER_ENTERING_WORLD"
function EnableOn(self, event, ...)
  if event == "QUEST_ACCEPTED" then
    local _, questID = ...
    return IsWorldQuest(questID) and not WORLD_QUESTS_BLACKLIST[questID]
  elseif event == "PLAYER_ENTERING_WORLD" then
    local inInstance, type = IsInInstance()
    if inInstance and type == "party" then
      return self:HasWorldQuest()
    end
  end
  return false
end

__DisableOnCondition__ "EQT_WORLDQUEST_REMOVED" "PLAYER_ENTERING_WORLD"
function DisableOn(self, event, ...)
  if event == "EQT_WORLDQUEST_REMOVED" then
    for _, _ in pairs(WORLD_QUESTS_CACHE) do return false end
  elseif event == "PLAYER_ENTERING_WORLD" then
    return not self:HasWorldQuest()
  end
  return  true
end



__SystemEvent__ "QUEST_POI_UPDATE" "QUEST_LOG_UPDATE" "QUEST_WATCH_LIST_CHANGED"
function WORLD_QUESTS_UPDATE()
  _M:UpdateWorldQuests()
end

__SystemEvent__()
function QUEST_ACCEPTED(self, questID)
  Debug("Accept quest id :  %i", questID)
  if not IsWorldQuest(questID) or WORLD_QUESTS_CACHE[questID] or WORLD_QUESTS_BLACKLIST[questID] then return end



  local worldQuest = _ObjectManager:Get(WorldQuest)
  worldQuest.id = questID

  _M:AddWorldQuestToCache(worldQuest)
  _M:UpdateWorldQuest(worldQuest)

  _WorldQuestBlock:AddWorldQuest(worldQuest)

  --print(IsWorldQuest, C_TaskQuest.GetQuestInfoByQuestID(questID))

  -- local isInArea, isOnMap, numObjectives, taskName, displayAsObjective = GetTaskInfo(questID)
  _WorldQuestBlock:Draw()


end

__SystemEvent__()
function QUEST_REMOVED(questID)
  if not IsWorldQuest(questID) then return end

  Debug("Remove world quest : %i", questID)
  _WorldQuestBlock:RemoveWorldQuest(questID)
  _M:RemoveWorldQuestFromCache(questID)
  _M:FireSystemEvent("EQT_WORLDQUEST_REMOVED")
  _WorldQuestBlock:Draw()
end

__Thread__()
function LoadWorldQuests(self)
  self:ClearWorldQuests()
  local numEntries, numQuests = GetNumQuestLogEntries()

  for i = 1, numEntries do
    local title, level, suggestedGroup, isHeader, isCollapsed, isComplete,
    frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI,
    isTask, isBounty, isStory, isHidden = GetQuestLogTitle(i)

    if isHidden then
      WORLD_QUESTS_BLACKLIST[questID] = true
    end

    if not isHidden and isTask and not WORLD_QUESTS_CACHE[questID] then
      local worldQuest = _ObjectManager:Get(WorldQuest)
      worldQuest.id = questID
      worldQuest.name = title
      Debug("Add WorldQuest : %s | %id", worldQuest.id, worldQuest.name)

      self:AddWorldQuestToCache(worldQuest)
      self:UpdateWorldQuest(worldQuest)

      _WorldQuestBlock:AddWorldQuest(worldQuest)
    end
  end
  _WorldQuestBlock:Draw()

end


function HasWorldQuest(self)
  for i = 1, GetNumQuestLogEntries() do
    if select(13, GetQuestLogTitle(i)) then
      return true
    end
  end
  return false
end

function AddWorldQuestToCache(self, worldQuest)
  WORLD_QUESTS_CACHE[worldQuest.id] = worldQuest
end

function RemoveWorldQuestFromCache(self, worldQuestID)
  WORLD_QUESTS_CACHE[worldQuestID] = nil
end

function UpdateWorldQuest(self, worldQuest)
  local isInArea, isOnMap, numObjectives, taskName, displayAsObjective = GetTaskInfo(worldQuest.id)
  worldQuest.isOnMap = true
  worldQuest.name = taskName

  local itemLink, itemTexture = GetQuestLogSpecialItemInfo(GetQuestLogIndexByID(worldQuest.id))
  if itemLink and itemTexture then
    local itemQuest  = worldQuest:GetQuestItem()
    itemQuest.link = itemLink
    itemQuest.texture = itemTexture
  end

  if numObjectives then
    worldQuest.numObjectives = numObjectives
    for index = 1, numObjectives do
      local text, type, finished = GetQuestObjectiveInfo(worldQuest.id, index, false)
      local objective = worldQuest:GetObjective(index)

      objective.isCompleted = finished
      objective.text = text

      if type == "progressbar" then
        local progress = GetQuestProgressBarPercent(worldQuest.id)
        objective:ShowProgress()
        objective:SetMinMaxProgress(0, 100)
        objective:SetProgress(progress)
        objective:SetTextProgress(string.format("%i%%", progress))
      else
        objective:HideProgress()
      end
    end
  end

end

function UpdateWorldQuests(self)
  for _, worldQuest in pairs(WORLD_QUESTS_CACHE) do
    self:UpdateWorldQuest(worldQuest)
  end
  -- @NOTE Add a draw here, check if it fix
end

function ClearWorldQuests(self)
  for _, worldQuest in pairs(WORLD_QUESTS_CACHE) do
      WORLD_QUESTS_CACHE[worldQuest.id] = nil
    _WorldQuestBlock:RemoveWorldQuest(worldQuest)
  end
end



-- QuestUtils_IsQuestWorldQuest
--[[
C_TaskQuest.GetDistanceSqToQuest
C_TaskQuest.GetQuestInfoByQuestID
C_TaskQuest.GetQuestLocation
C_TaskQuest.GetQuestProgressBarInfo
C_TaskQuest.GetQuestTitleByQuestID
	C_TaskQuest.GetQuestTimeLeftMinutes
C_TaskQuest.GetQuestsForPlayerByMapID
C_TaskQuest.GetQuestsForPlayerByMapID
C_TaskQuest.IsActive--]]
--C_TaskQuest.GetQuestTimeLeftMinutes
